@@ -13,7 +13,7 @@ from ophyd.areadetector import (DetectorBase, CamBase,
                                 EpicsSignalWithRBV as SignalWithRBV)
 from ophyd import (Signal, EpicsSignal, EpicsSignalRO, DerivedSignal)
 
-from ophyd import (Component as Cpt, FormattedComponent as FC,
+from ophyd import (Device, Component as Cpt, FormattedComponent as FC,  # noqa: F401
                    DynamicDeviceComponent as DDC)
 from ophyd.areadetector.plugins import PluginBase
 from ophyd.areadetector.filestore_mixins import FileStorePluginBase
@@ -115,6 +115,9 @@ class Xspress3FileStore(FileStorePluginBase, HDF5Plugin):
             # this needs a fail-safe, RE will now hang forever here
             # as we eat all SIGINT to ensure that cleanup happens in
             # orderly manner.
+            # If we are here this is a sign that we have not configured the xs3
+            # correctly and it is expecting to capture more points than it
+            # was triggered to take.
             while self.capture.get() == 1:
                 i += 1
                 if (i % 50) == 0:
@@ -122,10 +125,14 @@ class Xspress3FileStore(FileStorePluginBase, HDF5Plugin):
                 time.sleep(0.1)
                 if i > 150:
                     logger.warning('Still capturing data .... giving up.')
+                    logger.warning('Check that the xspress3 is configured to take the right '
+                                   'number of frames '
+                                   f'(it is trying to take {self.parent.settings.num_images.get()})')
                     self.capture.put(0)
                     break
 
         except KeyboardInterrupt:
+            self.capture.put(0)
             logger.warning('Still capturing data .... interrupted.')
 
         return super().unstage()
@@ -148,6 +155,8 @@ class Xspress3FileStore(FileStorePluginBase, HDF5Plugin):
         self.settings.acquire.put(0, wait=True)
 
         total_points = self.parent.total_points.get()
+        if total_points < 1:
+            raise RuntimeError("You must set the total points")
         spec_per_point = self.parent.spectra_per_point.get()
         total_capture = total_points * spec_per_point
 
@@ -195,9 +204,9 @@ class Xspress3FileStore(FileStorePluginBase, HDF5Plugin):
                                                self.file_name.get(),
                                                self.file_number.get())
 
-        if not self.file_path_exists.value:
+        if not self.file_path_exists.get():
             raise IOError("Path {} does not exits on IOC!! Please Check"
-                          .format(self.file_path.value))
+                          .format(self.file_path.get()))
 
         logger.debug('Inserting the filestore resource: %s', self._fn)
         self._generate_resource({})
@@ -484,7 +493,7 @@ class Xspress3Detector(DetectorBase):
 
     external_trig = Cpt(Signal, value=False,
                         doc='Use external triggering')
-    total_points = Cpt(Signal, value=2,
+    total_points = Cpt(Signal, value=-1,
                        doc='The total number of points to acquire overall')
     spectra_per_point = Cpt(Signal, value=1,
                             doc='Number of spectra per point')
@@ -610,6 +619,7 @@ class XspressTrigger(BlueskyInterface):
     def unstage(self):
         ret = super().unstage()
         self._acquisition_signal.clear_sub(self._acquire_changed)
+        self._status = None
         return ret
 
     def _acquire_changed(self, value=None, old_value=None, **kwargs):
@@ -625,7 +635,6 @@ class XspressTrigger(BlueskyInterface):
             raise RuntimeError("not staged")
 
         self._status = DeviceStatus(self)
-        self.settings.erase.put(1)
         self._acquisition_signal.put(1, wait=False)
         trigger_time = ttime.time()
 
